@@ -33,21 +33,22 @@
 #endif // USE_OPENGL
 #include <stdio.h>
 
+#include <spice/protocol.h>
+#include "common/region.h"
+
+#ifdef USE_OPENGL
+#include "common/gl_utils.h"
+#include "red_pixmap_gl.h"
+#endif // USE_OPENGL
+
 #include "red_window.h"
 #include "utils.h"
 #include "debug.h"
 #include "platform.h"
 #include "x_platform.h"
 #include "pixels_source_p.h"
-#include <spice/protocol.h>
-#include "region.h"
-#ifdef USE_OPENGL
-#include "gl_utils.h"
-#include "red_pixmap_gl.h"
-#endif // USE_OPENGL
 #include "x_icon.h"
 
-
 #define X_RETRIES 10
 #define X_RETRY_DELAY_MICRO (1000 * 100)
 #define RAISE_RETRIES X_RETRIES
@@ -186,6 +187,9 @@ enum EvdevKeyCode {
     EVDEV_KEYCODE_PAGE_DOWN,
     EVDEV_KEYCODE_INSERT,
     EVDEV_KEYCODE_DELETE,
+    EVDEV_KEYCODE_MUTE = 121,
+    EVDEV_KEYCODE_VOLUME_DOWN = 122,
+    EVDEV_KEYCODE_VOLUME_UP = 123,
     EVDEV_KEYCODE_PAUSE = 127,
     EVDEV_KEYCODE_HANGUL = 130,
     EVDEV_KEYCODE_HANGUL_HANJA,
@@ -456,6 +460,9 @@ static void init_evdev_map()
 {
     #define KEYMAP(key_code, red_key)  keycode_map[EVDEV_##key_code] = red_key
     INIT_MAP;
+    KEYMAP(KEYCODE_MUTE, REDKEY_MUTE);
+    KEYMAP(KEYCODE_VOLUME_DOWN, REDKEY_VOLUME_DOWN);
+    KEYMAP(KEYCODE_VOLUME_UP, REDKEY_VOLUME_UP);
     #undef KEYMAP
 }
 
