@@ -20,12 +20,12 @@
 #endif
 
 #include <arpa/inet.h>
-
 #include <vscard_common.h>
-#include "server/reds.h"
-#include "server/char_device.h"
-#include "server/red_channel.h"
-#include "server/smartcard.h"
+
+#include "reds.h"
+#include "char_device.h"
+#include "red_channel.h"
+#include "smartcard.h"
 
 #define SMARTCARD_MAX_READERS 10
 
@@ -65,7 +65,7 @@ static struct Readers {
     SpiceCharDeviceInstance* sin[SMARTCARD_MAX_READERS];
 } g_smartcard_readers = {0, {NULL}};
 
-static SpiceCharDeviceInstance* smartcard_readers_get_unattached();
+static SpiceCharDeviceInstance* smartcard_readers_get_unattached(void);
 static SpiceCharDeviceInstance* smartcard_readers_get(uint32_t reader_id);
 static int smartcard_char_device_add_to_readers(SpiceCharDeviceInstance *sin);
 static void smartcard_char_device_attach(
@@ -77,7 +77,7 @@ static void smartcard_char_device_on_message_from_device(
     SmartCardDeviceState *state, VSCMsgHeader *header);
 static void smartcard_on_message_from_device(
     RedChannelClient *rcc, VSCMsgHeader *vheader);
-static SmartCardDeviceState* smartcard_device_state_new();
+static SmartCardDeviceState* smartcard_device_state_new(void);
 static void smartcard_device_state_free(SmartCardDeviceState* st);
 static void smartcard_init(void);
 
@@ -101,7 +101,7 @@ void smartcard_char_device_wakeup(SpiceCharDeviceInstance *sin)
         if (actual_length > state->buf_size) {
             state->buf_size = MAX(state->buf_size*2, actual_length + sizeof(VSCMsgHeader));
             state->buf = spice_realloc(state->buf, state->buf_size);
-            ASSERT(state->buf != NULL);
+            spice_assert(state->buf != NULL);
         }
         if (state->buf_used - sizeof(VSCMsgHeader) < actual_length) {
             continue;
@@ -135,7 +135,7 @@ void smartcard_char_device_on_message_from_device(
     }
     /* We pass any VSC_Error right now - might need to ignore some? */
     if (state->reader_id == VSCARD_UNDEFINED_READER_ID && vheader->type != VSC_Init) {
-        red_printf("error: reader_id not assigned for message of type %d", vheader->type);
+        spice_printerr("error: reader_id not assigned for message of type %d", vheader->type);
     }
     if (state->rcc) {
         sent_header = spice_memdup(vheader, sizeof(*vheader) + vheader->length);
@@ -176,11 +176,11 @@ static int smartcard_char_device_add_to_readers(SpiceCharDeviceInstance *char_de
 
 static SpiceCharDeviceInstance *smartcard_readers_get(uint32_t reader_id)
 {
-    ASSERT(reader_id < g_smartcard_readers.num);
+    spice_assert(reader_id < g_smartcard_readers.num);
     return g_smartcard_readers.sin[reader_id];
 }
 
-static SpiceCharDeviceInstance *smartcard_readers_get_unattached()
+static SpiceCharDeviceInstance *smartcard_readers_get_unattached(void)
 {
     int i;
     SmartCardDeviceState* state;
@@ -195,7 +195,7 @@ static SpiceCharDeviceInstance *smartcard_readers_get_unattached()
     return NULL;
 }
 
-static SmartCardDeviceState* smartcard_device_state_new()
+static SmartCardDeviceState* smartcard_device_state_new(void)
 {
     SmartCardDeviceState *st;
 
@@ -284,15 +284,15 @@ static void smartcard_channel_release_msg_rcv_buf(RedChannelClient *rcc,
                                                   uint32_t size,
                                                   uint8_t *msg)
 {
-    red_printf("freeing %d bytes", size);
+    spice_printerr("freeing %d bytes", size);
     free(msg);
 }
 
 static void smartcard_channel_send_data(RedChannelClient *rcc, SpiceMarshaller *m,
                                         PipeItem *item, VSCMsgHeader *vheader)
 {
-    ASSERT(rcc);
-    ASSERT(vheader);
+    spice_assert(rcc);
+    spice_assert(vheader);
     red_channel_client_init_send_data(rcc, SPICE_MSG_SMARTCARD_DATA, item);
     spice_marshaller_add_ref(m, (uint8_t*)vheader, sizeof(VSCMsgHeader));
     if (vheader->length > 0) {
@@ -428,8 +428,7 @@ static void smartcard_channel_write_to_reader(VSCMsgHeader *vheader)
     uint32_t n;
     uint32_t actual_length = vheader->length;
 
-    ASSERT(vheader->reader_id >= 0 &&
-           vheader->reader_id <= g_smartcard_readers.num);
+    spice_assert(vheader->reader_id <= g_smartcard_readers.num);
     sin = g_smartcard_readers.sin[vheader->reader_id];
     sif = SPICE_CONTAINEROF(sin->base.sif, SpiceCharDeviceInterface, base);
     /* protocol requires messages to be in network endianess */
@@ -439,7 +438,7 @@ static void smartcard_channel_write_to_reader(VSCMsgHeader *vheader)
     n = sif->write(sin, (uint8_t*)vheader,
                    actual_length + sizeof(VSCMsgHeader));
     // TODO - add ring
-    ASSERT(n == actual_length + sizeof(VSCMsgHeader));
+    spice_assert(n == actual_length + sizeof(VSCMsgHeader));
 }
 
 static int smartcard_channel_handle_message(RedChannelClient *rcc,
@@ -454,7 +453,7 @@ static int smartcard_channel_handle_message(RedChannelClient *rcc,
         return red_channel_client_handle_message(rcc, size, type, msg);
     }
 
-    ASSERT(size == vheader->length + sizeof(VSCMsgHeader));
+    spice_assert(size == vheader->length + sizeof(VSCMsgHeader));
     switch (vheader->type) {
         case VSC_ReaderAdd:
             smartcard_add_reader(rcc, msg + sizeof(VSCMsgHeader));
@@ -479,7 +478,7 @@ static int smartcard_channel_handle_message(RedChannelClient *rcc,
     }
 
     if (vheader->reader_id >= g_smartcard_readers.num) {
-        red_printf("ERROR: received message for non existent reader: %d, %d, %d", vheader->reader_id,
+        spice_printerr("ERROR: received message for non existent reader: %d, %d, %d", vheader->reader_id,
             vheader->type, vheader->length);
         return FALSE;
     }
@@ -501,6 +500,9 @@ static void smartcard_connect(RedChannel *channel, RedClient *client,
     rcc = red_channel_client_create(sizeof(RedChannelClient), channel, client, stream,
                                     num_common_caps, common_caps,
                                     num_caps, caps);
+    if (!rcc) {
+        return;
+    }
     red_channel_client_ack_zero_messages_window(rcc);
 }
 
@@ -512,13 +514,10 @@ SmartCardChannel *g_smartcard_channel;
 
 static void smartcard_init(void)
 {
-    ChannelCbs channel_cbs;
-    ClientCbs client_cbs;
+    ChannelCbs channel_cbs = { NULL, };
+    ClientCbs client_cbs = { NULL, };
 
-    ASSERT(!g_smartcard_channel);
-
-    memset(&channel_cbs, 0, sizeof(channel_cbs));
-    memset(&client_cbs, 0, sizeof(client_cbs));
+    spice_assert(!g_smartcard_channel);
 
     channel_cbs.config_socket = smartcard_channel_client_config_socket;
     channel_cbs.on_disconnect = smartcard_channel_on_disconnect;
@@ -536,7 +535,7 @@ static void smartcard_init(void)
                                              &channel_cbs);
 
     if (!g_smartcard_channel) {
-        red_error("failed to allocate Inputs Channel");
+        spice_error("failed to allocate Inputs Channel");
     }
 
     client_cbs.connect = smartcard_connect;
