@@ -18,7 +18,7 @@
 #ifndef __SMART_CARD_H__
 #define __SMART_CARD_H__
 
-#include "server/spice-experimental.h"
+#include "spice-experimental.h"
 
 // Maximal length of APDU
 #define APDUBufSize 270
