@@ -1,4 +1,4 @@
-#include <strings.h>
+#include <config.h>
 #include <sys/select.h>
 #include <spice.h>
 #include "basic_event_loop.h"
@@ -16,4 +16,3 @@ int main(void)
 
     return 0;
 }
-
